import type Hls from "hls.js";

let cachedHls: null | Promise<typeof Hls | false> | typeof Hls | false = null;

const loadHls = () =>
  cachedHls ?? (cachedHls = (
    // @ts-ignore
    +"HLS POLYFILL", import("hls.js")
  ).then((m, y = m.default) => cachedHls = y.isSupported() && y));

const createVideoPlayer = async (videoElement: HTMLVideoElement) => {
  const sources = videoElement.querySelectorAll("source");
  for (let i = 0; i < sources.length; i++) {
    const { src, type } = sources[i];
    if (videoElement.canPlayType(type)) {
      videoElement.src = src;
      return;
    }

    if (type === "application/x-mpegURL" && await loadHls()) {
      const hls = new (cachedHls as typeof Hls)();
      hls.loadSource(src);
      hls.attachMedia(videoElement);
      return;
    }
  }
};

document.querySelectorAll("video").forEach(createVideoPlayer);
